//+------------------------------------------------------------------+
//|                    Jim Sloman's natural market mirror & SD bands |
//|                                            ocn nmm & SD hist.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DimGray
#property indicator_color2  LimeGreen
#property indicator_color3  Green
#property indicator_color4  DeepSkyBlue
#property indicator_style1  STYLE_DOT
#property indicator_style2  STYLE_DOT

//
//
//
//
//

extern int    NMM_Period             = 40;
extern int    NMM_Price              = PRICE_CLOSE;
extern int    TEMA_Period            = 10;
extern bool   Show_StandardDeviation = true;
extern int    SD_Period              = 30;
extern double SD_Up                  = 2.0;
extern double SD_Down                = 2.0;
extern bool   Show_HistoricalExtreme = true;
extern int    History_LookBack       = 15;

//
//
//
//
//

double nmm[];
double zeroLine[];
double sd[];
double hist[];
double tBuffer[][4];
double alpha;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,zeroLine); SetIndexLabel(0,NULL);
   SetIndexBuffer(1,hist); SetIndexDrawBegin(1,NMM_Period+SD_Period);
   SetIndexBuffer(2,sd);   SetIndexDrawBegin(2,NMM_Period+SD_Period);
   SetIndexBuffer(3,nmm);
    
   //
   //
   //
   //
   //

   string PriceType;
      switch(NMM_Price)
      {
         case PRICE_CLOSE:    PriceType = "Close";    break;  // 0
         case PRICE_OPEN:     PriceType = "Open";     break;  // 1
         case PRICE_HIGH:     PriceType = "High";     break;  // 2
         case PRICE_LOW:      PriceType = "Low";      break;  // 3
         case PRICE_MEDIAN:   PriceType = "Median";   break;  // 4
         case PRICE_TYPICAL:  PriceType = "Typical";  break;  // 5
         case PRICE_WEIGHTED: PriceType = "Weighted"; break;  // 6
      }      
  
      //
      //
      //
      //
      //
   
      TEMA_Period = MathMax(TEMA_Period,1);
      NMM_Period  = MathMax(NMM_Period ,1);
           alpha = 2.0 /(1.0 + TEMA_Period);
           
   IndicatorShortName ("nmm ("+NMM_Period+","+PriceType+","+TEMA_Period+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define iPrc 3

//
//
//
//
//

int start()
{
   int    counted_bars=IndicatorCounted();
   int    i,k,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;
         if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);

   //
   //
   //
   //
   //

   for(i=limit, r=Bars-i-1; i >= 0; i--,r++)
   {
      double rawPrice  = iMA(NULL,0,1,0,MODE_SMA,NMM_Price,i);
      double currPrice = iTema(rawPrice,i);
         if (currPrice > 0)
               tBuffer[r][iPrc] = MathLog(currPrice);
         else  tBuffer[r][iPrc] = 0.00;
      
         //
         //
         //
         //
         //
         
         nmm[i]      = iNmmFunction(1,i);
         zeroLine[i] = 0.00;
         
         //
         //
         //
         //
         //

         if (Show_StandardDeviation)
         {         
            sd[i] = 0;
            double avg = iSma(nmm,SD_Period,i);
            double deviation = iDeviation(nmm,avg,SD_Period,i);
               if (nmm[i] > 0) sd[i] = avg+deviation*SD_Up;
               if (nmm[i] < 0) sd[i] = avg-deviation*SD_Down;
         }
         
         //
         //
         //
         //
         //
         
         if (Show_HistoricalExtreme)
         {
            hist[i]  = hist[i+1];
               if (nmm[i] >  0) hist[i] = MathMax(0,nmm[ArrayMaximum(nmm,History_LookBack,i)]);
               if (nmm[i] <  0) hist[i] = MathMin(0,nmm[ArrayMinimum(nmm,History_LookBack,i)]);
               if (nmm[i] == 0) hist[i] = 0;
         }               
   }
   
   //
   //
   //
   //
   //
   
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//    natural market mirror
//
//
//

double iNmmFunction(int startPeriod, int tPos)
{
   int    endPeriod = startPeriod*NMM_Period;
   int    pos       = Bars-tPos-1;
   double sum       = 0.00;
   double currPrice = tBuffer[pos][iPrc];
      
   for (int i= startPeriod; i <= endPeriod; i += startPeriod)
           sum += (currPrice-tBuffer[pos-i][iPrc])/MathSqrt(i);
   return((sum/NMM_Period)*1000.00);
}

//
//
//
//
//

double iTema(double price,int pos,int sbuf=0)
{
   int i  = Bars-pos-1;
   int ia = sbuf+0;
   int ib = sbuf+1;
   int ic = sbuf+2;
   
   if (i < 1)
      {
         tBuffer[i][ia] = price;
         tBuffer[i][ib] = price;
         tBuffer[i][ic] = price;
      }
   else
      {
         tBuffer[i][ia] = tBuffer[i-1][ia]+alpha*(price         -tBuffer[i-1][ia]);
         tBuffer[i][ib] = tBuffer[i-1][ib]+alpha*(tBuffer[i][ia]-tBuffer[i-1][ib]);
         tBuffer[i][ic] = tBuffer[i-1][ic]+alpha*(tBuffer[i][ib]-tBuffer[i-1][ic]);
      }
   return(3*tBuffer[i][ia] - 3*tBuffer[i][ib] + tBuffer[i][ic]);
}

//
//
//
//
//

double iDeviation(double& array[],double dMA, int period, int pos)
{
   double dSum = 0;
         for(int i=0; i<period; i++,pos++) dSum += (array[pos]-dMA)*(array[pos]-dMA);
   return(MathSqrt(dSum/period));
}

//
//
//
//
//

double iSma(double& array[], int period, int pos)
{
   double sum = 0;
         for(int i=0; i<period; i++,pos++) sum += array[pos];
   return(sum/period);
}

